/***********************************************************************
/* anemometer.c
/* 
/* Anemometer sample.  As the Anemometer rotates the rotating portion 
/* contains multiple magnets that close a reed switch contact.  The 
/* number of closures within a defined time period is used to calculate  
/* the wind speed.
/*
/* One rotation occurs for every 4 switch closures.
/*
/* The transfer function for MPH and m/s: 
/*    MPH = ( Hz * 0.857 ) + 0.725
/*    m/s = ( Hz * 0.383 ) + 0.324  
/*    km/h = (( Hz * 0.383 ) + 0.324) * 3.6
/*
/* Setup timer 0 for 3 second timeout period. RA2 is setup to interrupt
/* on falling edge output of Anemometer. The pulses from the Anemometer
/* are counted during 3 second periods defined by timer 0.  Each 3 second
/* period is used to determine the wind "gust".  Multiple 3 second samples 
/* are smoothed through an FIR filter. 
/* 
/* Any changes to the time period necessitate changes to the wind sample
/* processing routine. 
/*
/*
/*
/*
/*
/***********************************************************************/

#include <stdio.h>
#include <delays.h>
#include <math.h>
#include <string.h>
#include <spi.h>

#include <p18l14k22.h>

#include "anemometer.h"

extern struct wind wind;

volatile unsigned int edge_count;
volatile unsigned int working_count;
volatile char edge_count_flag;

const int timer_init = 0x48E4;

void Init_timer0(void) {
// Timer 0 initialized so that the rollover occurs at 3 second intervals.
// With a prescale of 1:128, rollover occurs after 46875dec counter increments.
// Initialize the counter value with 18660dec (0x48E4).

  T0CONbits.T0CS = 0;    //internal instruction cycle used to increment timer.  
  T0CONbits.T08BIT = 0;  // 16 bit wide
  T0CONbits.PSA = 0;     // Use the prescalar
  T0CONbits.T0PS0 = 0;    // and set to 1:128
  T0CONbits.T0PS1 = 1;    
  T0CONbits.T0PS2 = 1;    

  TMR0H = 0x48;          // Must write to timer 0 high buffer first
  TMR0L = 0xE4;          // Write to timer 0 low buffer will write to register H:L bytes.

  return;
}

void Windsample(void) {

//  const float   coefficient = 0.2;   // 5 samples
//  const char     samples = 5;
//  const float   coefficient = 0.1;   // 10 samples
//  const char     samples = 10;
  const float   coefficient = 0.05;   // 20 samples
  const char     samples = 20;
  const float   recip_coeff = 20;

  const float     sample_period = 3;    // 3 seconds (the 3 second period depends upon Timer 0. See int_monitor.c)

  unsigned int   gust_count;
  unsigned int   gust_count_hz;
  unsigned int   count_upper_hz;

  float          wind_out;
  float          gust_out;

  char i,x;


  // init variables
  edge_count = 0;
  edge_count_flag = 1;
  wind.count_ave = 0;
  wind.units = mps;      // default meters/sec

  // Initialize the data array and preload the moving average
  for(i=0;i<samples;i++) {
    wind.count_samples[i] = coefficient;
    wind.count_ave += wind.count_samples[i];
  }

  i=0;
  
  while(1) {

    for(i ; i<samples ; i++) {
      // while true, let timer and edge counting continue
      // edge_count_flag is 
      while(edge_count_flag);  

      // If the edge_count_flag is clear, the timer has overflowed and caused
      // an interrupt.  Process the data.
      gust_count = working_count;

      // FIR filter. Keep track of the old sample in array prior to overwrite
      // This will be subtracted from the moving average negating the need to 
      // add up all the data array values every pass

      wind.count_ave -= wind.count_samples[i];  
      wind.count_samples[i] = (float)working_count * coefficient;
      wind.count_ave += wind.count_samples[i];

      // reset the for loop count variable,
      // "for" loop increments i, when "if" is true i = 255 will rollover to i=0 on next increment
      if(i>=(samples-1)) i=255;   
                                  
      // reset the edge count flag
      edge_count_flag = 1;

      // The transfer function for MPH and m/s: 
      //  MPH = ( Hz * 0.857 ) + 0.725
      //  m/s = ( Hz * 0.383 ) + 0.324
      //  km/hr = (( Hz * 0.383 ) + 0.324) * 3.6
      
      // Gust maximum is the maximum wind speed recorded in the 20 sample data array.
      // This is reported when queried
      for(x=0;x<samples;x++) {
         Nop();
         if((float)gust_count < (wind.count_samples[x]*recip_coeff)) {
            gust_count = (unsigned int)(wind.count_samples[x]*recip_coeff);
         }
         Nop();
      }
//NEW CODE///      // Evaluate for maximum gust within the sample array
//NEW CODE///      // The "gust" is taken from the highest wind sample of the wind data array every
//NEW CODE///      // pass through the function, which is every 3 seconds (the 3 second timer period).
//NEW CODE///      for(x=0;x<=samples;x++) {

//      gust_count_hz = gust_count/sample_period;
//      count_upper_hz = (unsigned int)wind.count_ave/sample_period;

      // Typecasting the float to unsigned int will remove the fractional part.
      // The integer result is rounded down, for example: 
      // 0.99 becomes 0, 1.23 becomes 1, 1.99 becomes 1, and 2.00 becomes 2
      // This decrease in resolution is acceptable. 

      if(!wind.units) {
         // wind speed is in mile/hr. Rely on type conversion for data
         wind_out = ((wind.count_ave/sample_period) * 0.857) + 0.725;
         gust_out = (((float)gust_count/sample_period) * 0.857) + 0.725;
         sprintf(wind.windspeed,"1W%03.3u mph speed %03.3u mph\n\r",(unsigned int)wind_out, (unsigned int)gust_out);
//         sprintf(wind.windspeed," gust %03.3u mph, speed %03.3u mph\n\r",(unsigned int)wind_out, (unsigned int)guxt_out);
      }
      else {
         // wind speed is in km/hr. Rely on type conversion for data
         wind_out = (((wind.count_ave/sample_period) * 0.383) + 0.324)* 3.6;
         gust_out = (((gust_count/sample_period) * 0.383) + 0.324)* 3.6;
         if(wind_out < 1.17 & gust_out < 1.17) wind_out = gust_out = 0.0;
         sprintf(wind.windspeed,"1W%03.3u%03.3u",(unsigned int)wind_out, (unsigned int)gust_out);
//         sprintf(wind.windspeed," gust %03.3u km/h, speed %03.3u km/h\n\r",(unsigned int)gust_out, (unsigned int)wind_out);
      }
    };  // End for( )
  };  // End while(1)
  
  return;
}

