/***********************************************************************
/*  Anemometer main.c
/*
/*  Anemometer sampling and data processing.  Continuously sample the 
/*  anemometer, process and parse the data, and respond to data queries.
/*  Main.c sets up interrupts and intializes ports.
/*
/*
/*
/***********************************************************************/
/* Compile options:  -ml (Large code model) */

#include <stdio.h>
#include <delays.h>
#include <usart.h>
#include <p18l14k22.h>

#include "main.h"
#include "serial.h"
#include "int_monitor.h"
#include "anemometer.h"

#pragma config WDTEN = OFF
#pragma config FOSC = IRC
#pragma config MCLRE = ON


// Interrupt handler is in int_monitor.c
#pragma code high_interrupt = 0x08
void high_isr (void)
{
   _asm goto High_isr_handler _endasm
}
#pragma code low_interrupt = 0x18
void low_isr (void)
{
   _asm goto Low_isr_handler _endasm
}

#pragma code  /* return to default code section */

// global container for the anemometer wind speed data
// structure defined in anemometer.h
struct wind wind;

void main(void)
{
  // Set oscillator to 8MHz
  OSCCON = 0x62;

  // Initialize ports
  Init_ports();

  // Init serial communications, RS232
  Serialcommservice();

  // Init timer 0 for 3 second periods (Fosc = 8MHz)
  Init_timer0();

  // Initialize interrupts
  Init_interrupts();

  // Enable timer 0 interrupt
  INTCONbits.TMR0IE = 1; 

  // Enable high and low priorty interrupts
  INTCONbits.GIEH = 1;
  INTCONbits.GIEL = 1;

  // Enable timer 0
  T0CONbits.TMR0ON = 1;
  INTCON3bits.INT2IE = 1;

  // Continuously sample the anemometer and calculate wind speed 
  Windsample();

  // If code is here, reset (application should never reach this code).
  Reset();

}


void Init_ports(void) {

// RA2, Input, External interrupt from anemometer
// RB5, Input, RX from host serial port
//      Initialization performed by the UART init routine
// RB7, Output, TX to host serial port
//      Initialization performed by the UART init routine

// RC0, Input, Xbee sleep status indicator (ON/SLEEP*, Xbee output, pin 13)
// RC7, Output, Control the Xbee sleep mode, active high (DTR*/SLEEP_RQ, Xbee input, pin 9)

  // Configure all ADC channels for digital I/O,
  ANSEL = 0;
  ANSELH = 0;

  // Initialize the ports, port pin direction and state.

  PORTAbits.RA2 = 0;

  PORTBbits.RB5 = 0;
  PORTBbits.RB7 = 0;

  PORTCbits.RC0 = 0;
  PORTCbits.RC7 = 0;

  TRISAbits.TRISA2 = 1;

  TRISBbits.TRISB5 = 1;
  TRISBbits.TRISB7 = 0;

  TRISCbits.TRISC0 = 1;
  TRISCbits.TRISC7 = 0;

  return;
}

void Init_interrupts(void) {

  // Enable interrupts priority
  RCONbits.IPEN = 1;
  
  // Make receive interrupt low priority, timer 0 high priority,
  //  and external interrupt 2 high priority
  IPR1bits.RCIP = 0;
  INTCON2bits.TMR0IP = 1;
  INTCON3bits.INT2IP = 1;

  // Clear interrupt flag
  INTCONbits.TMR0IF = 0;
  INTCON3bits.INT2IF = 0;

  // Interrupt 2 on falling edge
  INTCON2bits.INTEDG2 = 0;

  return;
}